# -*- coding: UTF-8 -*-
# @Time    : 2022/9/24 20:39
# @File    : ocr.py
# @Author  : jian<jian@mltalker.com>
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
import copy
import os
from mmocr.apis import MMOCRInferencer


class Ocr(object):
  def __init__(self, det_config='dbnetpp', recog_config='SAR', device='cuda:0'):
      self.det_config = det_config
      self.recog_config = recog_config
      self.device = device
      self.ocr_infer = MMOCRInferencer(det=self.det_config, rec=self.recog_config, device=self.device)

  def __call__(self, *args, **kwargs):
    result = self.ocr_infer(args[0], return_vis=False)
    if len(result['predictions']) == 0:
        # det_polygons, det_scores, rec_texts, rec_scores
        return None, None, None, None

    predictions = result['predictions'][0]
    det_polygons = None
    det_scores = None
    if 'det_polygons' in predictions:
        det_polygons = predictions['det_polygons']
    if 'det_scores' in predictions:
        det_scores = predictions['det_scores']

    rec_texts = None
    rec_scores = None
    if 'rec_texts' in predictions:
        rec_texts = predictions['rec_texts']
    if 'rec_scores' in predictions:
        rec_scores = predictions['rec_scores']

    return det_polygons, det_scores, rec_texts, rec_scores



# -*- coding: UTF-8 -*-
# @Time    : 2022/9/24 13:53
# @File    : detector.py
# @Author  : jian<jian@mltalker.com>
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
import numpy as np
from mmdet.apis import init_detector, inference_detector
import os
import subprocess
import sys


class Detector(object):
  def __init__(self, config_file, checkpoint_file=None, thres=0.4, device='cuda:0'):
    self.thres = thres
    if not config_file.endswith('.py'):
      config_file = f'{config_file}.py'
    config_name = config_file.split('.')[0]
    model_folder = os.path.join(os.environ['HOME'], '.antgo', 'models', 'mmdetection', config_name)
    if not os.path.exists(model_folder):
      os.makedirs(model_folder)

    model_file = os.path.join(model_folder, config_file)
    if not os.path.exists(model_file):
      subprocess.check_call(['mim', 'download', 'mmdet', '--config', config_name, '--dest', model_folder])

    if checkpoint_file is None:
      for f in os.listdir(model_folder):
        if f.endswith('.pth'):
          checkpoint_file = f
          break
    assert(checkpoint_file is not None)

    config_file = os.path.join(model_folder, config_file)
    checkpoint_file = os.path.join(model_folder, checkpoint_file)
    self.model = init_detector(config_file, checkpoint_file, device=device)  # or device='cuda:0'

  def __call__(self, *args, **kwargs):
    result = inference_detector(self.model, args[0])
    pred_instances_score = result.pred_instances.scores.to('cpu').numpy()
    pred_instances_label = result.pred_instances.labels.to('cpu').numpy()
    pred_instances_bbox = result.pred_instances.bboxes.to('cpu').numpy()
    valid_pos = np.where(pred_instances_score > self.thres)
    valid_pred_bbox = pred_instances_bbox[valid_pos]
    valid_pred_score = pred_instances_score[valid_pos]
    valid_pred_label = pred_instances_label[valid_pos]
    valid_pred_bbox = np.concatenate([valid_pred_bbox, valid_pred_score.reshape(-1, 1)], -1)
    return valid_pred_bbox, valid_pred_label




import os
import json
import yaml


def check_device_info(args):
    if args.ssh:
        # 远程执行命令
        ssh_config_file = os.path.join(os.environ['HOME'], '.config', 'antgo', 'ssh-submit-config.yaml')
        with open(ssh_config_file, 'r') as fp:
          ssh_config_info = yaml.safe_load(fp)

        print(f'ACTIVE(REMOTE BY SSH): {ssh_config_info["config"]["ip"]}')
        os.system(f'ssh {ssh_config_info["config"]["username"]}@{ssh_config_info["config"]["ip"]} nvidia-smi')
    elif args.local:
        os.system('nvidia-smi')
    elif args.k8s:
        print(f'k8s not support now')
    else:
        os.system('nvidia-smi')



import os
import cv2
import numpy as np


def perspective_transform(pt, t):
    new_pt = np.array([pt[0], pt[1], 1.0]).T
    new_pt = np.dot(t, new_pt)
    new_pt[:2] = new_pt[:2] / new_pt[-1]
    return new_pt[:2]


def simple_syn_sample(image_generator, obj_generator, min_scale=0.5, max_scale=0.8):
    for image, obj_info in zip(image_generator, obj_generator):
        # obj_info {'image': , 'points': , 'mask': }

        # step 1. 图像调整到适合尺寸
        random_scale = np.random.random() * (max_scale - min_scale) + min_scale
        image_h, image_w, _ = image.shape

        object_image = obj_info['image']
        obj_h, obj_w, _ = object_image.shape

        min_size = max(image_w, image_h) * random_scale
        obj_scale = min_size / max(obj_h, obj_w)

        if obj_w * obj_scale > image_w * random_scale or obj_h * obj_scale > image_h * random_scale:
            if obj_w * obj_scale > image_w * random_scale:
                obj_scale = image_w * random_scale / obj_w

            if obj_h * obj_scale > image_h * random_scale:
                obj_scale = image_h * random_scale / obj_h

        object_image = cv2.resize(object_image, dsize=(int(obj_w * obj_scale), int(obj_h * obj_scale)))
        obj_h, obj_w = object_image.shape[:2]

        if 'points' in obj_info:
           obj_info['points'] = obj_info['points'] * obj_scale

        object_paste_mask = np.ones((obj_h, obj_w)).astype(np.uint8)
        if object_image.shape[-1] == 4:
            object_paste_mask = object_paste_mask[:,:, 3]

        if 'mask' in obj_info:
            obj_info['mask'] = cv2.resize(obj_info['mask'], dsize=(obj_w, obj_h))

        # step 2. 随机透射变换
        # 生成随机变换矩阵 3x3
        p1 = np.array([[0,0], [obj_w-1, 0], [0, obj_h-1], [obj_w-1, obj_h-1]]).astype(np.float32)
        tgt_p1 = [np.random.randint(0, obj_w//4), np.random.randint(0, obj_h//4)]
        tgt_p2 = [np.random.randint(obj_w-obj_w//4, obj_w), np.random.randint(0, obj_h//4)]
        tgt_p3 = [np.random.randint(0, obj_w//4), np.random.randint(obj_h-obj_h//4, obj_h)]
        tgt_p4 = [np.random.randint(obj_w-obj_w//4, obj_w), np.random.randint(obj_h-obj_h//4, obj_h)]
        p2 = np.array([tgt_p1, tgt_p2, tgt_p3, tgt_p4]).astype(np.float32)
        M = cv2.getPerspectiveTransform(p1, p2)
        object_image = cv2.warpPerspective(object_image, M, (obj_w, obj_h))
        object_paste_mask = cv2. warpPerspective(object_paste_mask, M, (obj_w, obj_h))
        if 'points' in obj_info:
            for point_i in range(obj_info['points'].shape[0]):
                obj_info['points'][point_i, :2] = perspective_transform(obj_info['points'][point_i, :2], M)

        if 'mask' in obj_info:
            obj_info['mask'] = \
                cv2.warpPerspective(
                    obj_info['mask'],
                    M, 
                    (obj_w, obj_h), 
                    borderMode=cv2.BORDER_CONSTANT,
                    borderValue=(255))

        # 合成
        paste_x = 0
        if image_w > obj_w:
            paste_x = np.random.randint(0, image_w - obj_w)
        paste_y = 0
        if image_h > obj_h:
            paste_y = np.random.randint(0, image_h - obj_h)

        print(f'image {image_w} {image_h}')
        print(f'obj {obj_w} {obj_h}')
        object_paste_mask_expand = np.expand_dims(object_paste_mask, -1)
        image[paste_y:paste_y+obj_h, paste_x:paste_x+obj_w] = image[paste_y:paste_y+obj_h, paste_x:paste_x+obj_w] * (1-object_paste_mask_expand) + object_image * object_paste_mask_expand

        if 'points' in obj_info:
            obj_info['points'] = obj_info['points'] + np.float32([[paste_x, paste_y]])

        if 'mask' in obj_info:
            mask = np.ones((image_h, image_w), dtype=np.uint8) * 255
            mask[paste_y:paste_y+obj_h, paste_x:paste_x+obj_w] = mask[paste_y:paste_y+obj_h, paste_x:paste_x+obj_w] * (1-object_paste_mask) + obj_info['mask'] * object_paste_mask
            obj_info['mask'] = mask

        # for joint_i, (x,y) in enumerate(obj_info['points']):
        #     x, y = int(x), int(y)
        #     image = cv2.circle(image, (x, y), radius=2, color=(0,0,255), thickness=1)

        # cv2.imwrite(f'./abcd.png', image)
        yield image, obj_info




from antgo.framework.helper.fileio.file_client import *
import os
import logging

def ls_from_aliyun(src_path):
    ali = AliBackend()
    if not src_path.startswith("ali://"):
        src_path = f'{"ali://"}{src_path}'
    
    for file_name in ali.ls(src_path):
        print(file_name)



import os
import logging
import subprocess
import yaml


def stop_ssh_running(action_name, id):
    ssh_submit_config_file = os.path.join(os.environ['HOME'], '.config', 'antgo', 'ssh-submit-config.yaml')
    if not os.path.exists(ssh_submit_config_file):
        logging.error('No ssh config.')
        return False

    with open(ssh_submit_config_file, encoding='utf-8', mode='r') as fp:
        ssh_config_info = yaml.safe_load(fp)

    username = ssh_config_info['config']['username']
    password = ssh_config_info['config']['password']
    ip = ssh_config_info['config']['ip']

    print(f'ACTIVE(REMOTE BY SSH): {ssh_config_info["config"]["ip"]}')
    os.system(f'ssh {ssh_config_info["config"]["username"]}@{ssh_config_info["config"]["ip"]} docker stop {id}')

def ls_ssh_running():
    ssh_submit_config_file = os.path.join(os.environ['HOME'], '.config', 'antgo', 'ssh-submit-config.yaml')
    if not os.path.exists(ssh_submit_config_file):
        logging.error('No ssh config.')
        return False

    with open(ssh_submit_config_file, encoding='utf-8', mode='r') as fp:
        ssh_config_info = yaml.safe_load(fp)
    
    username = ssh_config_info['config']['username']
    password = ssh_config_info['config']['password']
    ip = ssh_config_info['config']['ip']

    print(f'ACTIVE(REMOTE BY SSH): {ssh_config_info["config"]["ip"]}')
    cmd = f'ssh {ssh_config_info["config"]["username"]}@{ssh_config_info["config"]["ip"]} docker ps'
    ret = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    if ret.returncode:
        logging.error("Couldnt get running info")

    running_info = ret.stdout.read()
    running_info = running_info.decode('utf-8')
    running_info = running_info.split('\n')
    if len(running_info) <= 1:
        print('No running info')
        return

    for i in range(1, len(running_info)):
        if running_info[i] == '':
            continue

        container_info = running_info[i].split(' ')
        container_id = container_info[0]
        container_name = container_info[-1]

        print(f'running id {container_id}, name {container_name}')


def log_ssh_running(action_name, id):
    ssh_submit_config_file = os.path.join(os.environ['HOME'], '.config', 'antgo', 'ssh-submit-config.yaml')
    if not os.path.exists(ssh_submit_config_file):
        logging.error('No ssh config.')
        return False

    with open(ssh_submit_config_file, encoding='utf-8', mode='r') as fp:
        ssh_config_info = yaml.safe_load(fp)

    username = ssh_config_info['config']['username']
    password = ssh_config_info['config']['password']
    ip = ssh_config_info['config']['ip']

    print(f'ACTIVE(REMOTE BY SSH): {ssh_config_info["config"]["ip"]}')
    os.system(f'ssh {ssh_config_info["config"]["username"]}@{ssh_config_info["config"]["ip"]} docker logs -f {id}')


def operate_on_running_status(action_name, args):
    if args.ssh:
        if action_name == 'stop':
            stop_ssh_running(action_name, args.id)
        elif action_name == 'ls':
            ls_ssh_running()
        elif action_name == 'log':
            log_ssh_running(action_name, args.id)
        else:
            logging.error("Only support stop/ls/log")
    else:
        logging.error("Only support operate for ssh task")



from antgo.framework.helper.fileio.file_client import *
import os
import logging


def upload_to_aliyun(target_folder, src_path):
    if not target_folder.startswith('ali://'):
        target_folder = f'ali://{target_folder}'
    
    if not os.path.exists(src_path):
        logging.error(f"{src_path} dont exist")
        return

    ali = AliBackend()
    ali.upload(target_folder, src_path)



# encoding=utf-8
# Time: 5/2/17
# File: __init__.py.py
# Author: jian<jian@mltalker.com>
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function

from .simplesyn import simple_syn_sample

__all__ = ['simple_syn_sample']